@@ -61,9 +61,39 @@ struct  _GpaPpdPaper {
 	gchar *code;
 	ArtPoint dimension;
 	ArtDRect region;
+
+	gboolean add_code;
 };
 
 
+#define GPA_TAG_PAPER_SET_CODE "PaperSetCode"
+
+/**
+ * gpa_ppd_papers_set_code_not_cleaned:
+ * @list: 
+ * 
+ * Ok, we tried to clean the code but we where unable to, just add all the setcode as a variable
+ * and set the setcode to "$VALUE-SetCode-"
+ * 
+ * Return Value: 
+ **/
+static gboolean
+gpa_ppd_papers_set_code_not_cleaned (GList *list_in, gchar **set_code, const gchar *start_tag, const gchar *end_tag)
+{
+	GpaPpdPaper *paper;
+	GList *list;
+		
+	*set_code = g_strdup_printf ("%s" GPA_TAG_PAPER_SET_CODE "%s", start_tag, end_tag);
+
+	list = list_in;
+	for (; list != NULL; list = list->next) {
+		paper = (GpaPpdPaper *)list->data;
+		paper->add_code = TRUE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 gpa_ppd_papers_get_set_code (GList *papers, gchar **set_code)
 {
@@ -90,8 +120,8 @@ gpa_ppd_papers_get_set_code (GList *papers, gchar **set_code)
 	}
 	
 	if (!gpa_ppd_simple_div_solve (ppd_div, start_tag, end_tag, set_code)) {
-		gpa_ppd_error ("Could not determine the set code for the Paper Options");
-		return FALSE;
+		/* Don't optimize, just add the paper code to each paper */
+		return gpa_ppd_papers_set_code_not_cleaned (papers, set_code, start_tag, end_tag);
 	}
 
 	/* NO need to free the strings pointed by labels, those are freed by
@@ -99,7 +129,7 @@ gpa_ppd_papers_get_set_code (GList *papers, gchar **set_code)
 	g_free (labels);
 	
 	debug (FALSE, "end");
-			
+
 	return TRUE;
 }
 
@@ -160,12 +190,12 @@ gpa_ppd_papers_get_query_code  (const gchar *code, gint code_size, gchar **query
 	tag = g_strdup_printf ("*?PageSize: \"");
 	tag_length = strlen (tag);
 	pos = gpa_tu_search_real (code, code_size,
-				      tag, tag_length, TRUE);
+				  tag, tag_length, TRUE);
+	/* We don't require query code, so if we didn't find it just return */
 	if (pos < 0) {
-		gpa_ppd_error ("Could not find \"%s\" in the PageSize code (4.6)\n",
-			       tag);
+		*query_code_ = NULL;
 		g_free (tag);
-		return FALSE;
+		return TRUE;
 	}
 
 	buffer = code + pos + tag_length;
@@ -176,6 +206,8 @@ gpa_ppd_papers_get_query_code  (const gchar *code, gint code_size, gchar **query
 
 	*query_code_ = query_code;
 
+	g_free (tag);
+	
 	return TRUE;
 }
 
@@ -234,8 +266,8 @@ gpa_ppd_paper_replace_dimension_with_values (const gchar *buffer_in,
 	return TRUE;
 }
 
-/* This function is messsy but it works. I have no clue on how to beautify it,
-   patches wellcomed */
+/* This function is _REALLY_ messsy but it works. I have no clue on how to beautify it,
+   patches wellcomed. Chema */
 static gboolean
 gpa_ppd_papers_clean_query_code (GList *papers, gchar **query_code_)
 {
@@ -258,8 +290,10 @@ gpa_ppd_papers_clean_query_code (GList *papers, gchar **query_code_)
 	gint n;
 
 	query_code = *query_code_;
+	if (query_code == NULL)
+		return TRUE;
+	
 	g_return_val_if_fail (papers != NULL, FALSE);
-	g_return_val_if_fail (query_code != NULL, FALSE);
 
 	/* 1. search for "dict"
 	   2. get the previous token and verify that it corresponds
@@ -281,12 +315,12 @@ gpa_ppd_papers_clean_query_code (GList *papers, gchar **query_code_)
 	/* 1 */
 	buffer = query_code;
 	buffer_length = strlen (query_code);
-	tag_length = strlen (GPA_PPD_TAG_DICT);
+	tag_length = strlen (" " GPA_PPD_TAG_DICT);
 	pos = gpa_tu_search_real (buffer, buffer_length,
-				      GPA_PPD_TAG_DICT, tag_length, TRUE);
+				  " " GPA_PPD_TAG_DICT, tag_length, TRUE);
 	if (pos < 0) {
 		gpa_ppd_error ("Could not find \"%s\" in query_code (4.7)",
-					GPA_PPD_TAG_DICT);
+			       " " GPA_PPD_TAG_DICT);
 		return FALSE;
 	}
 	/* 2 */
@@ -455,16 +489,16 @@ gpa_ppd_papers_get_region (GpaPpdInfo *ppd_info, GList *papers)
 	gint tag_length;
 	gint pos;
 	gint offset;
+	gboolean dump = FALSE;
 	
 	debug (FALSE, "");
 
 	g_return_val_if_fail (ppd_info != NULL, FALSE);
 	g_return_val_if_fail (papers != NULL, FALSE);
 
-	buffer = ppd_info->ppd;
-	buffer_size = ppd_info->ppd_size;
-	
 	for (list = papers; list != NULL; list = list->next) {
+		buffer = ppd_info->ppd;
+		buffer_size = ppd_info->ppd_size;
 		paper = list->data;
 		g_return_val_if_fail (paper->name != NULL, FALSE);
 		tag = g_strdup_printf ("*ImageableArea %s", paper->full_name);
@@ -496,27 +530,56 @@ gpa_ppd_papers_get_region (GpaPpdInfo *ppd_info, GList *papers)
 			gpa_ppd_error ("Could not find %s (1.4)\n", tag);
 			return FALSE;
 		}
-		offset+=2;
+		if (!gpa_tu_token_next_till (buffer, buffer_size, &offset, '"')) {
+			gpa_ppd_error ("Could not find %s (1.4.2)\n", tag);
+			return FALSE;
+		}
+
+		/* 1 */
 		token = gpa_tu_token_next_dup (buffer, buffer_size, &offset);
+		if (dump)
+			g_print ("Imageable area 1 :%s\n", token);
+		if (gpa_tu_string_contains_newline (token))
+			goto gpa_ppd_paper_get_region_error;
 		paper->region.x0 = atof (token);
 		g_free (token);
+
+		/* 2 */
 		token = gpa_tu_token_next_dup (buffer, buffer_size, &offset);
+		if (dump)
+			g_print ("Imageable area 2 :%s\n", token);
+		if (gpa_tu_string_contains_newline (token))
+			goto gpa_ppd_paper_get_region_error;
 		paper->region.y0 = atof (token);
 		g_free (token);
+
+		/* 3 */
 		token = gpa_tu_token_next_dup (buffer, buffer_size, &offset);
+		if (dump)
+			g_print ("Imageable area 3 :%s\n", token);
+		if (gpa_tu_string_contains_newline (token))
+			goto gpa_ppd_paper_get_region_error;
 		paper->region.x1 = atof (token);
 		g_free (token);
+
+		/* 4 */
 		token = gpa_tu_token_next_dup_till (buffer, buffer_size, &offset, '"');
+		if (dump)
+			g_print ("Imageable area 4 :%s\n", token);
+		if (gpa_tu_string_contains_newline (token))
+			goto gpa_ppd_paper_get_region_error;
 		paper->region.y1 = atof (token);
 		g_free (token);
 
-		g_return_val_if_fail (paper->region.x0  != 0, FALSE);
-		g_return_val_if_fail (paper->region.y0  != 0, FALSE);
-		g_return_val_if_fail (paper->region.x1  != 0, FALSE);
-		g_return_val_if_fail (paper->region.y1  != 0, FALSE);
 	}
 
 	return TRUE;
+
+gpa_ppd_paper_get_region_error:
+	gpa_error ("An error was encountered while trying to determine the ImageableArea for \"%s\"",
+		   paper->full_name);
+	return FALSE;
+	
 }
 
 static gboolean
@@ -531,16 +594,16 @@ gpa_ppd_papers_get_dimension (GpaPpdInfo *ppd_info, GList *papers)
 	gint tag_length;
 	gint pos;
 	gint offset;
+	gboolean dump = FALSE;
 	
 	debug (FALSE, "");
 
 	g_return_val_if_fail (ppd_info != NULL, FALSE);
 	g_return_val_if_fail (papers != NULL, FALSE);
 
-	buffer = ppd_info->ppd;
-	buffer_size = ppd_info->ppd_size;
-	
 	for (list = papers; list != NULL; list = list->next) {
+		buffer = ppd_info->ppd;
+		buffer_size = ppd_info->ppd_size;
 		paper = list->data;
 		g_return_val_if_fail (paper->name != NULL, FALSE);
 		tag = g_strdup_printf ("*PaperDimension %s", paper->full_name);
@@ -549,7 +612,7 @@ gpa_ppd_papers_get_dimension (GpaPpdInfo *ppd_info, GList *papers)
 			pos = gpa_tu_search_real (buffer, buffer_size,
 						      tag, tag_length, TRUE);
 			if (pos == -1) {
-				gpa_ppd_error ("Could not find \"%s\" (1.5)\n", tag);
+				gpa_ppd_error ("Could not find \"%s\" (1.5.2)-->%s<--\n", tag, buffer);
 				g_free (tag);
 				return FALSE;
 			}
@@ -573,19 +636,39 @@ gpa_ppd_papers_get_dimension (GpaPpdInfo *ppd_info, GList *papers)
 			gpa_ppd_error ("Could not find %s (1.6)\n", tag);
 			return FALSE;
 		}
-		offset+=2;
+		if (!gpa_tu_token_next_till (buffer, buffer_size, &offset, '"')) {
+			gpa_ppd_error ("Could not find %s (1.6.2)\n", tag);
+			return FALSE;
+		}
+		
+		/* 1 */
 		token = gpa_tu_token_next_dup (buffer, buffer_size, &offset);
+		if (dump)
+			g_print ("Paper Dimesion 1 :%s\n", token);
+		if (gpa_tu_string_contains_newline (token))
+			goto gpa_ppd_paper_get_dimension_error;
 		paper->dimension.x = atof (token);
 		g_free (token);
-		token = gpa_tu_token_next_dup_till (buffer, buffer_size, &offset, '"');
+
+		/* 2 */
+		token = gpa_tu_token_next_dup (buffer, buffer_size, &offset);
+		if (dump)
+			g_print ("Paper Dimesion 2 :%s\n", token);
+		if (gpa_tu_string_contains_newline (token))
+			goto gpa_ppd_paper_get_dimension_error;
 		paper->dimension.y = atof (token);
 		g_free (token);
-
+		
 		g_return_val_if_fail (paper->dimension.x != 0, FALSE);
 		g_return_val_if_fail (paper->dimension.y != 0, FALSE);
 	}
 
 	return TRUE;
+
+gpa_ppd_paper_get_dimension_error:
+	gpa_error ("An error was encountered while trying to determine the PaperDimension for \"%s\"",
+		   paper->full_name);
+	return FALSE;
 }
 
 
@@ -623,9 +706,10 @@ gpa_ppd_get_papers (const gchar *code, gint code_size)
 		buffer = buffer + pos + tag_length;
 		buffer_size = buffer_size - pos - tag_length;
 		offset = 0;
+		
 		/* Get & verify the paper by tokenizing it */
 		token = gpa_tu_token_next_dup_till (buffer, buffer_size,
-						&offset, ':');
+						    &offset, ':');
 
 		pos = gpa_tu_search_real (token, strlen (token),
 					      "/", 1, TRUE);
@@ -637,12 +721,14 @@ gpa_ppd_get_papers (const gchar *code, gint code_size)
 			paper_name = g_strndup (token, pos);
 			g_free (token);
 		}
-		offset++;
+		if (buffer [offset] == ' ')
+			offset++;
+		
 		if (buffer [offset] != '\"') {
 			offset = 0;
 			buffer_size = code_size - (buffer-code);
 			token = gpa_tu_token_next_dup_till_newline (buffer,	buffer_size,
-								&offset);
+								    &offset);
 			gpa_ppd_error ("Expected \"\"\" not found in -->%s<-- (2.2)",
 				       token);
 			g_free (token);
@@ -661,6 +747,7 @@ gpa_ppd_get_papers (const gchar *code, gint code_size)
 		paper->name = paper_name;
 		paper->code = paper_code;
 		paper->full_name = paper_full_name;
+		paper->add_code = FALSE;
 		list = g_list_prepend (list, paper);
 	}
 
@@ -706,6 +793,9 @@ gpa_ppd_get_option_from_paper (GpaPpdPaper *paper, GpaOptions *parent)
 	g_hash_table_insert (hash, GPA_TAG_REGION_X1, region_x1);
 	g_hash_table_insert (hash, GPA_TAG_REGION_Y1, region_y1);
 
+	if (paper->add_code)
+		g_hash_table_insert (hash, GPA_TAG_PAPER_SET_CODE, paper->code);
+		
 	option->values = hash;
 	
 	g_free (id);
@@ -763,21 +853,31 @@ gpa_ppd_dup_ui_code (GpaPpdInfo *ppd_info, gint *size, const gchar *ui_name)
 	g_return_val_if_fail (ppd_info != NULL, NULL);
 	g_return_val_if_fail (ui_name != NULL, NULL);
 
-	tag = g_strdup_printf ("*OpenUI *%s: PickOne", ui_name);
+	tag = g_strdup_printf ("*OpenUI *%s", ui_name);
 	tag_length = strlen (tag);
 
 	buffer = ppd_info->ppd;
 	buffer_size = ppd_info->ppd_size;
 	
 	pos_ini = gpa_tu_search_real (buffer, buffer_size,
-					  tag, tag_length, TRUE);
+				      tag, tag_length, TRUE);
 
 	if (pos_ini < 0) {
 		gpa_ppd_error ("Could not find %s (1.1)", tag);
 		return NULL;
 	}
 
+	/* Advance till we find a newline */
+	while (buffer [pos_ini] != '\n' &&
+	       buffer [pos_ini] != '\r')
+		pos_ini ++;
+	while (buffer [pos_ini] == '\n' ||
+	       buffer [pos_ini] == '\r')
+		pos_ini ++;
+
 	g_free (tag);
+
+	
 	tag = g_strdup_printf ("*CloseUI: *%s", ui_name);
 	tag_length = strlen (tag);
 	buffer += pos_ini;
@@ -839,8 +939,9 @@ gpa_ppd_add_paper_options (GpaPpdInfo *ppd_info)
 		return FALSE;
 	if (!gpa_ppd_papers_get_query_code   (ui_code, ui_code_size, &query_code))
 		return FALSE;
-	if (!gpa_ppd_papers_clean_query_code (papers, &query_code))
-		return FALSE;
+	if (!gpa_ppd_papers_clean_query_code (papers, &query_code)) {
+		gpa_ppd_warning ("The query code could not be cleaned. We will just include all the code here\n");
+	}
 
 	options = gpa_options_new (ppd_info->model,
 				   "Paper Size",
